@@ -6,7 +6,7 @@
  * \author Urs Fleisch
  * \date 25 Sep 2005
  *
- * Copyright (C) 2005-2018  Urs Fleisch
+ * Copyright (C) 2005-2023  Urs Fleisch
  *
  * This file is part of Kid3.
  *
@@ -40,12 +40,12 @@
 #include "genres.h"
 #include "modeliterator.h"
 #include "saferename.h"
-#include "fileproxymodel.h"
+#include "taggedfilesystemmodel.h"
 
 /**
  * Constructor.
  *
- * @param idx index in file proxy model
+ * @param idx index in tagged file system model
  */
 TaggedFile::TaggedFile(const QPersistentModelIndex& idx)
   : m_index(idx), m_truncation(0), m_modified(false), m_marked(false)
@@ -54,21 +54,21 @@ TaggedFile::TaggedFile(const QPersistentModelIndex& idx)
     m_changedFrames[tagNr] = 0;
     m_changed[tagNr] = false;
   }
-  Q_ASSERT(m_index.model()->metaObject() == &FileProxyModel::staticMetaObject);
-  if (const FileProxyModel* model = getFileProxyModel()) {
+  Q_ASSERT(m_index.model()->metaObject() == &TaggedFileSystemModel::staticMetaObject);
+  if (const TaggedFileSystemModel* model = getTaggedFileSystemModel()) {
     m_newFilename = model->fileName(m_index);
     m_filename = m_newFilename;
   }
 }
 
 /**
- * Get file proxy model.
- * @return file proxy model.
+ * Get tagged file model.
+ * @return tagged file model.
  */
-const FileProxyModel* TaggedFile::getFileProxyModel() const
+const TaggedFileSystemModel* TaggedFile::getTaggedFileSystemModel() const
 {
   // The validity of this cast is checked in the constructor.
-  return static_cast<const FileProxyModel*>(m_index.model());
+  return static_cast<const TaggedFileSystemModel*>(m_index.model());
 }
 
 /**
@@ -78,7 +78,7 @@ const FileProxyModel* TaggedFile::getFileProxyModel() const
  */
 QString TaggedFile::getDirname() const
 {
-  if (const FileProxyModel* model = getFileProxyModel()) {
+  if (const TaggedFileSystemModel* model = getTaggedFileSystemModel()) {
     return model->filePath(m_index.parent());
   }
   return QString();
@@ -114,7 +114,7 @@ void TaggedFile::setFilenameFormattedIfEnabled(QString fn)
  */
 void TaggedFile::updateCurrentFilename()
 {
-  if (const FileProxyModel* model = getFileProxyModel()) {
+  if (const TaggedFileSystemModel* model = getTaggedFileSystemModel()) {
     const QString newName = model->fileName(m_index);
     if (!newName.isEmpty() && m_filename != newName) {
       if (m_newFilename == m_filename) {
@@ -132,7 +132,7 @@ void TaggedFile::updateCurrentFilename()
  */
 QString TaggedFile::currentFilePath() const
 {
-  if (const FileProxyModel* model = getFileProxyModel()) {
+  if (const TaggedFileSystemModel* model = getTaggedFileSystemModel()) {
     return model->filePath(m_index);
   }
   return QString();
@@ -264,14 +264,22 @@ void TaggedFile::undoRevertChangedFilename()
  * Mark tag as changed.
  *
  * @param tagNr tag number
- * @param type type of changed frame
+ * @param extendedType type of changed frame
  */
-void TaggedFile::markTagChanged(Frame::TagNumber tagNr, Frame::Type type)
+void TaggedFile::markTagChanged(Frame::TagNumber tagNr,
+                                const Frame::ExtendedType& extendedType)
 {
+  Frame::Type type = extendedType.getType();
   m_changed[tagNr] = true;
   if (static_cast<unsigned>(type) < sizeof(m_changedFrames[tagNr]) * 8) {
     m_changedFrames[tagNr] |= (1ULL << type);
   }
+  if (type == Frame::FT_Other) {
+    const QString internalName = extendedType.getInternalName();
+    if (!internalName.isEmpty()) {
+      m_changedOtherFrameNames[tagNr].insert(internalName);
+    }
+  }
   updateModifiedState();
 }
 
@@ -282,17 +290,67 @@ void TaggedFile::markTagChanged(Frame::TagNumber tagNr, Frame::Type type)
 void TaggedFile::markTagUnchanged(Frame::TagNumber tagNr) {
   m_changed[tagNr] = false;
   m_changedFrames[tagNr] = 0;
+  m_changedOtherFrameNames[tagNr].clear();
   clearTrunctionFlags(tagNr);
   updateModifiedState();
 }
 
 /**
- * Set the mask of the frame types changed in tag.
+ * Get the types of the changed frames in a tag.
  * @param tagNr tag number
- * @param mask mask of frame types
+ * @return types of changed frames.
  */
-void TaggedFile::setChangedFrames(Frame::TagNumber tagNr, quint64 mask) {
-  m_changedFrames[tagNr] = mask;
+QList<Frame::ExtendedType> TaggedFile::getChangedFrames(
+    Frame::TagNumber tagNr) const {
+  QList<Frame::ExtendedType> types;
+  if (tagNr < Frame::Tag_NumValues) {
+    const QSet<QString> changedOtherFrameNames = m_changedOtherFrameNames[tagNr];
+    const quint64 changedFrames = m_changedFrames[tagNr];
+    quint64 mask;
+    int i;
+    for (i = Frame::FT_FirstFrame, mask = 1ULL;
+         i <= Frame::FT_LastFrame;
+         ++i, mask <<= 1) {
+      if (changedFrames & mask) {
+        types.append(Frame::ExtendedType(
+                       static_cast<Frame::Type>(i), QString()));
+      }
+    }
+    if (!changedOtherFrameNames.isEmpty()) {
+      for (const QString& name : changedOtherFrameNames) {
+        types.append(Frame::ExtendedType(Frame::FT_Other, name));
+      }
+    } else if (changedFrames & (1ULL << Frame::FT_Other)) {
+      types.append(Frame::ExtendedType(Frame::FT_Other, QString()));
+    }
+    if (changedFrames & (1ULL << Frame::FT_UnknownFrame)) {
+      types.append(Frame::ExtendedType());
+    }
+  }
+  return types;
+}
+
+/**
+ * Set the types of the changed frames in a tag.
+ * @param tagNr tag number
+ * @param types types of changed frames
+ */
+void TaggedFile::setChangedFrames(Frame::TagNumber tagNr,
+                                  const QList<Frame::ExtendedType>& types) {
+  quint64& mask = m_changedFrames[tagNr];
+  QSet<QString>& changedOtherFrameNames = m_changedOtherFrameNames[tagNr];
+  mask = 0;
+  changedOtherFrameNames.clear();
+  for (const auto& extendedType : types) {
+    Frame::Type type = extendedType.getType();
+    mask |= 1ULL << type;
+    if (type == Frame::FT_Other) {
+      const QString internalName = extendedType.getInternalName();
+      if (!internalName.isEmpty()) {
+        changedOtherFrameNames.insert(internalName);
+      }
+    }
+  }
   m_changed[tagNr] = mask != 0;
   updateModifiedState();
 }
@@ -309,8 +367,8 @@ void TaggedFile::updateModifiedState()
   modified = modified || m_newFilename != m_filename;
   if (m_modified != modified) {
     m_modified = modified;
-    if (const FileProxyModel* model = getFileProxyModel()) {
-      const_cast<FileProxyModel*>(model)->notifyModificationChanged(
+    if (const TaggedFileSystemModel* model = getTaggedFileSystemModel()) {
+      const_cast<TaggedFileSystemModel*>(model)->notifyModificationChanged(
             m_index, m_modified);
     }
   }
@@ -329,8 +387,8 @@ void TaggedFile::updateModifiedState()
 void TaggedFile::notifyModelDataChanged(bool priorIsTagInformationRead) const
 {
   if (isTagInformationRead() != priorIsTagInformationRead) {
-    if (const FileProxyModel* model = getFileProxyModel()) {
-      const_cast<FileProxyModel*>(model)->notifyModelDataChanged(m_index);
+    if (const TaggedFileSystemModel* model = getTaggedFileSystemModel()) {
+      const_cast<TaggedFileSystemModel*>(model)->notifyModelDataChanged(m_index);
     }
   }
 }
@@ -346,8 +404,8 @@ void TaggedFile::notifyTruncationChanged(bool priorTruncation) const
 {
   bool currentTruncation = m_truncation != 0;
   if (currentTruncation != priorTruncation) {
-    if (const FileProxyModel* model = getFileProxyModel()) {
-      const_cast<FileProxyModel*>(model)->notifyModelDataChanged(m_index);
+    if (const TaggedFileSystemModel* model = getTaggedFileSystemModel()) {
+      const_cast<TaggedFileSystemModel*>(model)->notifyModelDataChanged(m_index);
     }
   }
 }
@@ -467,12 +525,14 @@ void TaggedFile::getTagsFromFilename(FrameCollection& frames, const QString& fmt
       const int braceExprLen = closingBracePos - percentIdx + prefixLen - 1;
       if (!useCustomCaptures) {
         QString capture(QLatin1String(
-                          (code == QLatin1String("track number") ||
-                           code == QLatin1String("date") ||
-                           code == QLatin1String("disc number") ||
-                           code == QLatin1String("bpm"))
-                          ? "(\\d{1,4})"
-                          : "([^-_\\./ ](?:[^/]*[^-_/ ])?)"));
+                          code == QLatin1String("track number")
+                          ? "([A-Za-z]?\\d+[A-Za-z]?)"
+                          : code == QLatin1String("date")
+                            ? "(\\d{1,4}[\\dT :-]*)"
+                            : (code == QLatin1String("disc number") ||
+                               code == QLatin1String("bpm"))
+                              ? "(\\d{1,4})"
+                              : "([^-_\\./ ](?:[^/]*[^-_/ ])?)"));
         pattern.replace(percentIdx, braceExprLen, capture);
         percentIdx += capture.length();
       } else {
@@ -484,6 +544,11 @@ void TaggedFile::getTagsFromFilename(FrameCollection& frames, const QString& fmt
     }
   }
 
+  if (!useCustomCaptures) {
+    // accept file names with spaces before the extension
+    pattern += QLatin1String("\\s*");
+  }
+
   // and finally a dot followed by 2 to 4 characters for the extension
   pattern += QLatin1String("\\..{2,4}$");
 
@@ -621,7 +686,7 @@ bool TaggedFile::renameFile() const
   const QString dirname = getDirname();
   const QString fnOld = currentFilename();
   const QString fnNew = getFilename();
-  auto model = const_cast<FileProxyModel*>(getFileProxyModel());
+  auto model = const_cast<TaggedFileSystemModel*>(getTaggedFileSystemModel());
 
   if (fnNew.toLower() == fnOld.toLower()) {
     // If the filenames only differ in case, the new file is reported to
@@ -713,6 +778,64 @@ int TaggedFile::splitNumberAndTotal(const QString& str, int* total)
 #endif
 }
 
+/**
+ * Fix up a key to be valid.
+ * If the key contains new line characters because it is coming from an ID3
+ * frame (e.g. "COMM - COMMENTS\nDescription"), the description part is taken.
+ * Illegal characters depending on @a tagType are removed.
+ *
+ * @param key key which might have invalid characters.
+ * @param tagType tag type
+ * @return key which can be used for tag type.
+ */
+QString TaggedFile::fixUpTagKey(const QString& key, TagType tagType)
+{
+  int len = key.length();
+  int i = key.indexOf(QLatin1Char('\n'));
+  if (i < 0) {
+    // key does not contain '\n' => 0..len
+    i = 0;
+  } else if (i >= len - 1) {
+    // '\n' at end of key => 0..len-1
+    i = 0;
+    --len;
+  } else {
+    // key contains '\n' at i => i+1..len
+    ++i;
+  }
+
+  // Allowed characters depending on tag type:
+  // TT_Vorbis: != '=' && >= 0x20 && <= 0x7D
+  // TT_Ape: >= 0x20 && <= 0x7E
+  QChar forbidden;
+  QChar firstAllowed;
+  QChar lastAllowed;
+  if (tagType == TT_Vorbis) {
+    forbidden = QLatin1Char('=');
+    firstAllowed = QLatin1Char('\x20');
+    lastAllowed = QLatin1Char('\x7d');
+  } else if (tagType == TT_Ape) {
+    firstAllowed = QLatin1Char('\x20');
+    lastAllowed = QLatin1Char('\x7e');
+  }
+
+  QString result;
+  result.reserve(len - i);
+  if (forbidden.isNull() && firstAllowed.isNull() && lastAllowed.isNull()) {
+    result = key.mid(i, len - i);
+  } else {
+    while (i < len) {
+      QChar ch = key.at(i);
+      if (ch != forbidden &&
+          ch >= firstAllowed && ch <= lastAllowed) {
+        result.append(ch);
+      }
+      ++i;
+    }
+  }
+  return result;
+}
+
 /**
  * Get the total number of tracks in the directory.
  *
@@ -1030,45 +1153,36 @@ void TaggedFile::setFrames(Frame::TagNumber tagNr,
           setFrame(tagNr, *it);
         } else {
           // The frame does not have an index
-          if (it->getType() == Frame::FT_Track) {
-            // Handle track with the basic method
-            setFrame(tagNr, *it);
+          // The frame has to be looked up and modified
+          if (!myFramesValid) {
+            getAllFrames(tagNr, myFrames);
+            myFramesValid = true;
+          }
+          auto myIt = myFrames.find(*it);
+          int myIndex = -1;
+          while (myIt != myFrames.end() && !(*it < *myIt) &&
+                 (myIndex = myIt->getIndex()) != -1) {
+            if (!replacedIndexes.contains(myIndex)) {
+              break;
+            }
+            myIndex = -1;
+            ++myIt;
+          }
+          if (myIndex != -1) {
+            replacedIndexes.insert(myIndex);
+            if (!myIt->isFuzzyEqual(*it)) {
+              Frame myFrame(*it);
+              myFrame.setIndex(myIndex);
+              setFrame(tagNr, myFrame);
+            }
           } else {
-            // The frame has to be looked up and modified
-            if (!myFramesValid) {
-              getAllFrames(tagNr, myFrames);
-              myFramesValid = true;
-            }
-            auto myIt = myFrames.find(*it);
-            int myIndex = -1;
-            while (myIt != myFrames.end() && !(*it < *myIt) &&
-                   (myIndex = myIt->getIndex()) != -1) {
-              if (!replacedIndexes.contains(myIndex)) {
-                break;
-              }
-              myIndex = -1;
-              ++myIt;
-            }
-            if (myIndex != -1) {
-              replacedIndexes.insert(myIndex);
-              if (myIt->getValue() != it->getValue() ||
-                  (!myIt->getFieldList().isEmpty() &&
-                   !it->getFieldList().isEmpty() &&
-                   !Frame::Field::fuzzyCompareFieldLists(myIt->getFieldList(),
-                                                         it->getFieldList()))) {
-                Frame myFrame(*it);
-                myFrame.setIndex(myIndex);
-                setFrame(tagNr, myFrame);
-              }
-            } else {
-              // Such a frame does not exist, add a new one.
-              if (!it->getValue().isEmpty() || !it->getFieldList().isEmpty()) {
-                Frame addedFrame(*it);
-                addFrame(tagNr, addedFrame);
-                Frame myFrame(*it);
-                myFrame.setIndex(addedFrame.getIndex());
-                setFrame(tagNr, myFrame);
-              }
+            // Such a frame does not exist, add a new one.
+            if (!it->getValue().isEmpty() || !it->getFieldList().isEmpty()) {
+              Frame addedFrame(*it);
+              addFrame(tagNr, addedFrame);
+              Frame myFrame(*it);
+              myFrame.setIndex(addedFrame.getIndex());
+              setFrame(tagNr, myFrame);
             }
           }
         }
