@@ -6,7 +6,7 @@
  * \author Urs Fleisch
  * \date 9 Jan 2003
  *
- * Copyright (C) 2003-2018  Urs Fleisch
+ * Copyright (C) 2003-2023  Urs Fleisch
  *
  * This file is part of Kid3.
  *
@@ -236,6 +236,7 @@ void BaseMainWindowImpl::onDirectoryOpened()
  */
 void BaseMainWindowImpl::saveOptions()
 {
+  savePlayToolBarConfig();
   m_self->saveConfig();
   m_form->saveConfig();
   m_app->saveConfig();
@@ -249,6 +250,36 @@ void BaseMainWindowImpl::readOptions()
   m_app->readConfig();
   m_self->readConfig();
   m_form->readConfig();
+  readPlayToolBarConfig();
+}
+
+void BaseMainWindowImpl::savePlayToolBarConfig()
+{
+#ifdef HAVE_QTMULTIMEDIA
+  GuiConfig& guiCfg = GuiConfig::instance();
+  if (m_playToolBar) {
+    guiCfg.setPlayToolBarVisible(m_playToolBar->isVisible());
+    guiCfg.setPlayToolBarArea(m_w->toolBarArea(m_playToolBar));
+  } else {
+    guiCfg.setPlayToolBarVisible(false);
+  }
+#endif
+}
+
+void BaseMainWindowImpl::readPlayToolBarConfig()
+{
+#ifdef HAVE_QTMULTIMEDIA
+  const GuiConfig& guiCfg = GuiConfig::instance();
+  if (guiCfg.playToolBarVisible()) {
+    showPlayToolBar();
+    if (m_playToolBar) {
+      int area = guiCfg.playToolBarArea();
+      if (area == Qt::BottomToolBarArea || area == Qt::TopToolBarArea) {
+        m_w->addToolBar(static_cast<Qt::ToolBarArea>(area), m_playToolBar);
+      }
+    }
+  }
+#endif
 }
 
 /**
@@ -346,7 +377,8 @@ void BaseMainWindowImpl::saveDirectory(bool updateGui)
   m_app->deleteAudioPlayer();
 #endif
 
-  const QStringList errorFiles = m_app->saveDirectory();
+  QStringList errorDescriptions;
+  const QStringList errorFiles = m_app->saveDirectory(&errorDescriptions);
 
   if (!errorFiles.empty()) {
     QStringList errorMsgs, notWritableFiles;
@@ -360,6 +392,8 @@ void BaseMainWindowImpl::saveDirectory(bool updateGui)
         errorMsgs.append(fileInfo.fileName());
       }
     }
+    errorMsgs = Kid3Application::mergeStringLists(errorMsgs, errorDescriptions,
+                                                  QLatin1String(": "));
     if (notWritableFiles.isEmpty()) {
       m_platformTools->errorList(
         m_w, tr("Error while writing file:\n"),
@@ -1217,11 +1251,8 @@ void BaseMainWindowImpl::showPlayToolBar()
       connect(m_playToolBar, &PlayToolBar::closed,
               m_app, &Kid3Application::deactivateMprisInterface);
 #endif
-#ifdef Q_OS_WIN32
-      // Phonon on Windows cannot play if the file is open.
       connect(m_playToolBar, &PlayToolBar::aboutToPlay,
-              m_app, &Kid3Application::closeFileHandle);
-#endif
+              m_app, &Kid3Application::onAboutToPlay);
     }
   }
   m_playToolBar->show();
@@ -1459,7 +1490,7 @@ void BaseMainWindowImpl::onEditFrameDialogFinished(int result)
       }
       if (m_editFrameTaggedFile->setFrame(m_editFrameTagNr, m_editFrame)) {
         m_editFrameTaggedFile->markTagChanged(m_editFrameTagNr,
-                                              m_editFrame.getType());
+                                              m_editFrame.getExtendedType());
       }
     }
   }
@@ -1548,7 +1579,7 @@ void BaseMainWindowImpl::renameFile()
           nullptr, tr("File Error"),
           tr("Error while renaming:\n") +
           tr("Rename %1 to %2 failed\n").arg(fileName, newFileName),
-          QMessageBox::Ok, Qt::NoButton);
+          QMessageBox::Ok, QMessageBox::NoButton);
 #endif
       }
     }
@@ -1716,6 +1747,7 @@ void BaseMainWindowImpl::startProgressMonitoring(
   m_progressTerminationHandler = terminationHandler;
   m_progressDisconnected = disconnectModel;
   m_progressStartTime = QDateTime::currentDateTime();
+  m_form->saveFileAndDirListConfig();
 }
 
 /**
@@ -1729,6 +1761,7 @@ void BaseMainWindowImpl::stopProgressMonitoring()
     if (m_progressDisconnected) {
       m_form->getDirList()->reconnectModel();
       m_form->getFileList()->reconnectModel();
+      m_form->readFileAndDirListConfig();
       m_form->getFileList()->expandAll();
     }
   }
