@@ -78,8 +78,10 @@ QString getArtistString(const QJsonArray& artists)
       }
       artist += fixUpArtist((map.contains(QLatin1String("name"))
                              ? map.value(QLatin1String("name"))
-                             : map.value(QLatin1String("artist"))
-                               .toObject().value(QLatin1String("name")))
+                             : map.contains(QLatin1String("displayName"))
+                               ? map.value(QLatin1String("displayName"))
+                               : map.value(QLatin1String("artist"))
+                                 .toObject().value(QLatin1String("name")))
                             .toString());
       join = (map.contains(QLatin1String("join"))
               ? map.value(QLatin1String("join"))
@@ -243,6 +245,21 @@ void addCredit(FrameCollection& frames, Frame::Type type, const QString& name)
   frames.setValue(type, value);
 }
 
+/**
+ * Extract the URL from "fullsize.__ref" of an image JSON object.
+ * @param imageValue image JSON value
+ * @return image URL if present, else null.
+ */
+QString extractUrlFromImageValue(const QJsonValue& imageValue) {
+  QRegularExpression sourceUrlRe(
+        QLatin1String("\"sourceUrl\"\\s*:\\s*\"([^\"]+)\""));
+  QString ref = imageValue.toObject()
+      .value(QLatin1String("fullsize")).toObject()
+      .value(QLatin1String("__ref")).toString();
+  auto match = sourceUrlRe.match(ref);
+  return match.hasMatch() ? match.captured(1) : QString();
+}
+
 /**
  * Stores information about extra artists.
  * The information can be used to add frames to the appropriate tracks.
@@ -284,8 +301,10 @@ private:
 ExtraArtist::ExtraArtist(const QJsonObject& obj) :
   m_name(fixUpArtist((obj.contains(QLatin1String("name"))
                       ? obj.value(QLatin1String("name"))
-                      : obj.value(QLatin1String("artist"))
-                        .toObject().value(QLatin1String("name"))).toString())),
+                      : obj.contains(QLatin1String("displayName"))
+                        ? obj.value(QLatin1String("displayName"))
+                        : obj.value(QLatin1String("artist"))
+                          .toObject().value(QLatin1String("name"))).toString())),
   m_role((obj.contains(QLatin1String("role"))
           ? obj.value(QLatin1String("role"))
           : obj.value(QLatin1String("creditRole"))).toString().trimmed())
@@ -396,6 +415,10 @@ void TrackInfo::addToFrames(FrameCollection& frames,
   if (additionalTags && !m_disc.isNull()) {
     frames.setValue(Frame::FT_Disc, m_disc);
   }
+  if (additionalTags && m_pos == 0 && !m_position.isEmpty()) {
+    // Support tracks which are not numeric, e.g. "A2"
+    frames.setValue(Frame::FT_Track, m_position);
+  }
   for (const auto& extraArtist : trackExtraArtists) {
     extraArtist.addToFrames(frames, m_position);
   }
@@ -408,10 +431,12 @@ void TrackInfo::addToFrames(FrameCollection& frames,
  * property when getting it from the HTML output
  * @param importer Discogs importer
  * @param trackDataModel track data model to update with imported data
+ * @param data optional top level data
  * @return true if at least one title was found.
  */
 bool parseJsonAlbumResults(const QJsonObject& map,
-    const DiscogsImporter* importer, TrackDataModel* trackDataModel)
+    const DiscogsImporter* importer, TrackDataModel* trackDataModel,
+    const QJsonObject& data = QJsonObject())
 {
   // releases have the format (JSON, simplified):
   // { "styles": ["Heavy Metal"],
@@ -460,8 +485,11 @@ bool parseJsonAlbumResults(const QJsonObject& map,
     // The genre can be found in "genre" or "style".
     // All genres found are checked for an ID3v1 number, starting with those
     // in the style field.
-    const auto genreList = map.value(QLatin1String("styles")).toArray() +
-        map.value(QLatin1String("genres")).toArray();
+    // Converted to QVariantList because adding two QJsonArray will add the
+    // second array as a single element to the first array.
+    const auto genreList =
+        map.value(QLatin1String("styles")).toArray().toVariantList() +
+        map.value(QLatin1String("genres")).toArray().toVariantList();
     QStringList genres, customGenres;
     for (const auto& val : genreList) {
       QString genre = val.toString().trimmed();
@@ -553,7 +581,9 @@ bool parseJsonAlbumResults(const QJsonObject& map,
     if (!frames.getTitle().isEmpty()) {
       titleFound = true;
     }
-    if (frames.getTrack() == 0) {
+    if (frames.getValue(Frame::FT_Track).isEmpty()) {
+      // Track as string is used instead of "frames.getTrack() == 0" to support
+      // tracks like "A2"
       frames.setTrack(trackNr);
     }
     if (atTrackDataListEnd) {
@@ -591,6 +621,20 @@ bool parseJsonAlbumResults(const QJsonObject& map,
 
   for (const auto& val : trackList) {
     auto track = val.toObject();
+    if (track.size() == 1 && track.contains(QLatin1String("__ref")) &&
+        !data.isEmpty()) {
+      const QJsonObject trackRef = data.value(
+            track.value(QLatin1String("__ref")).toString()).toObject();
+      if (trackRef.contains(QLatin1String("title"))) {
+        track = trackRef;
+      }
+    }
+
+    // Do not include heading tracks
+    if (track.value(QLatin1String("trackType")).toString() ==
+        QLatin1String("HEADING")) {
+      continue;
+    }
 
     TrackInfo trackInfo(track);
 
@@ -740,7 +784,7 @@ void DiscogsImporter::HtmlImpl::parseFindResults(const QByteArray& searchStr)
   // <a class="search_result_title " href="/Amon-Amarth-The-Avenger/release/761529-Amon-Amarth-The-Avenger" data-followable="true">The Avenger</a>
   QString str = QString::fromUtf8(searchStr);
   QRegularExpression idTitleRe(QLatin1String(
-      "<a href=\"/artist/[^>]+?>([^<]+?)</a>[^-]*?-"
+      "href=\"/artist/[^>]+?>([^<]+?)</a>[^-]*?-"
       "\\s*?<a class=\"search_result_title[ \"]+?href=\"/([^/]*?/?release)/"
       "([0-9]+-[^\"]+?)\"[^>]*?>([^<]+?)</a>(.*?card_actions)"),
        QRegularExpression::DotMatchesEverythingOption);
@@ -812,6 +856,21 @@ void DiscogsImporter::HtmlImpl::parseAlbumResults(const QByteArray& albumStr)
             }
           }
 
+          // There are multiple images in the embedded JSON,
+          // try to find the correct one.
+          if (!release.isEmpty()) {
+            for (auto it = release.constBegin(); it != release.constEnd(); ++it) {
+              QString imageRef;
+              if (it.key().startsWith(QLatin1String("images")) &&
+                  !(imageRef = it.value().toObject()
+                    .value(QLatin1String("edges")).toArray().first().toObject()
+                    .value(QLatin1String("node")).toObject()
+                    .value(QLatin1String("__ref")).toString()).isEmpty()) {
+                imgUrl = extractUrlFromImageValue(data.value(imageRef));
+              }
+            }
+          }
+
           for (auto it = data.constBegin(); it != data.constEnd(); ++it) {
             if (it.key().startsWith(QLatin1String("Release:"))) {
               QJsonValue releaseValue;
@@ -823,15 +882,7 @@ void DiscogsImporter::HtmlImpl::parseAlbumResults(const QByteArray& albumStr)
               }
             } else if (it.key().startsWith(QLatin1String("Image:"))) {
               if (imgUrl.isEmpty()) {
-                QRegularExpression sourceUrlRe(
-                      QLatin1String("\"sourceUrl\"\\s*:\\s*\"([^\"]+)\""));
-                QString ref = it.value().toObject()
-                    .value(QLatin1String("fullsize")).toObject()
-                    .value(QLatin1String("__ref")).toString();
-                auto match = sourceUrlRe.match(ref);
-                if (match.hasMatch()) {
-                  imgUrl = match.captured(1);
-                }
+                imgUrl = extractUrlFromImageValue(it.value());
               }
             }
           }
@@ -841,7 +892,8 @@ void DiscogsImporter::HtmlImpl::parseAlbumResults(const QByteArray& albumStr)
                              QJsonArray({QJsonObject({{QLatin1String("uri"),
                                                        imgUrl}})}));
             }
-            if (parseJsonAlbumResults(release, m_importer, trackDataModel())) {
+            if (parseJsonAlbumResults(release, m_importer, trackDataModel(),
+                                      data)) {
               return;
             }
           }
